@@ -15,12 +15,15 @@
  * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#include <stdbool.h>
+
 #ifndef LTTNG_SESSIOND_AGENT_THREAD_H
 #define LTTNG_SESSIOND_AGENT_THREAD_H
 
 #ifdef HAVE_LIBLTTNG_UST_CTL
 
 void *agent_thread_manage_registration(void *data);
+bool agent_tracing_is_enabled(void);
 
 #else /* HAVE_LIBLTTNG_UST_CTL */
 
@@ -29,6 +32,11 @@ void *agent_thread_manage_registration(void *data)
 {
 	return NULL;
 }
+static inline
+bool agent_tracing_is_enabled(void)
+{
+	return false;
+}
 
 #endif /* HAVE_LIBLTTNG_UST_CTL */
 
